"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TextField_1 = require("../../../src/feed/field/TextField");
const IntField_1 = require("../../../src/feed/field/IntField");
const TimeField_1 = require("../../../src/feed/field/TimeField");
const DateField_1 = require("../../../src/feed/field/DateField");
const SingleRecordFile_1 = require("../../../src/feed/file/SingleRecordFile");
const BooleanField_1 = require("../../../src/feed/field/BooleanField");
const Record_1 = require("../../../src/feed/record/Record");
const util_1 = require("util");
/**
 * Short format DTD date (e.g. 31/05/2017)
 */
class ALFDate extends DateField_1.DateField {
    constructor(start, nullable = false) {
        super(start, nullable);
    }
    /**
     * Turn an ALF formatted date into a MySQL formatted date
     */
    parse(value) {
        return `${value.substr(-4)}-${value.substr(3, 2)}-${value.substr(0, 2)}`;
    }
}
/**
 * The ALF file has a bizarre CSV like format with optional fields so some specific processing is required.
 */
class ALFRecord {
    constructor() {
        this.fieldMap = {
            "mode": ["M"],
            "origin": ["O"],
            "destination": ["D"],
            "duration": ["T"],
            "start_time": ["S"],
            "end_time": ["E"],
            "priority": ["P"],
            "start_date": ["F"],
            "end_date": ["U"],
            "monday": ["R", 0],
            "tuesday": ["R", 1],
            "wednesday": ["R", 2],
            "thursday": ["R", 3],
            "friday": ["R", 4],
            "saturday": ["R", 5],
            "sunday": ["R", 6]
        };
        this.name = "additional_fixed_link";
        this.indexes = [];
        this.key = [];
        this.fields = {
            "mode": new TextField_1.VariableLengthText(1, 10),
            "origin": new TextField_1.TextField(2, 3),
            "destination": new TextField_1.TextField(3, 3),
            "duration": new IntField_1.IntField(4, 3, false, []),
            "start_time": new TimeField_1.TimeField(5, 4, false),
            "end_time": new TimeField_1.TimeField(6, 4, false),
            "priority": new IntField_1.IntField(7, 1),
            "start_date": new ALFDate(8, true),
            "end_date": new ALFDate(9, true),
            "monday": new BooleanField_1.BooleanField(10),
            "tuesday": new BooleanField_1.BooleanField(11),
            "wednesday": new BooleanField_1.BooleanField(12),
            "thursday": new BooleanField_1.BooleanField(13),
            "friday": new BooleanField_1.BooleanField(14),
            "saturday": new BooleanField_1.BooleanField(15),
            "sunday": new BooleanField_1.BooleanField(16)
        };
        this.orderedInserts = false;
    }
    /**
     * Split the CSV string and then split the values into key pairs.
     *
     * M=TRANSFER,O=LBG,D=WAT,T=25,S=0001,E=2359,P=6,F=29/08/2017,U=01/09/2017,R=0111100
     */
    extractValues(line) {
        const entries = line
            .trim()
            .split(",")
            .map(field => field.split("="));
        const csvMap = new Map(entries);
        const values = { id: null };
        const action = Record_1.RecordAction.Insert;
        for (const [name, field] of Object.entries(this.fields)) {
            const [fieldKey, position] = this.fieldMap[name];
            const rawValue = csvMap.get(fieldKey);
            if ((0, util_1.isNullOrUndefined)(rawValue)) {
                values[name] = null;
            }
            else {
                const value = !(0, util_1.isNullOrUndefined)(position) ? rawValue.charAt(position) : rawValue;
                values[name] = field.extract(value);
            }
        }
        return { action, values, keysValues: values };
    }
}
const ALF = new SingleRecordFile_1.SingleRecordFile(new ALFRecord());
exports.default = ALF;
//# sourceMappingURL=ALF.js.map