"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportFeedCommand = void 0;
const AdmZip = require("adm-zip");
const MySQLSchema_1 = require("../database/MySQLSchema");
const path = require("path");
const MySQLTable_1 = require("../database/MySQLTable");
const memoize = require("memoized-class-decorator");
const fs = require("fs-extra");
const MultiRecordFile_1 = require("../feed/file/MultiRecordFile");
const MySQLStream_1 = require("../database/MySQLStream");
const byline = require("byline");
const streamToPromise = require("stream-to-promise");
const getExt = filename => path.extname(filename).slice(1).toUpperCase();
const readFile = filename => byline.createStream(fs.createReadStream(filename, "utf8"));
/**
 * Imports one of the feeds
 */
class ImportFeedCommand {
    constructor(db, files, tmpFolder) {
        this.db = db;
        this.files = files;
        this.tmpFolder = tmpFolder;
    }
    get fileArray() {
        return Object.values(this.files);
    }
    /**
     * Do the import and then shut down the connection pool
     */
    async run(argv) {
        try {
            await this.doImport(argv[3]);
        }
        catch (err) {
            console.error(err);
        }
        return this.end();
    }
    /**
     * Extract the zip, set up the schema and do the inserts
     */
    async doImport(filePath) {
        console.log(`Extracting ${filePath} to ${this.tmpFolder}`);
        fs.emptyDirSync(this.tmpFolder);
        new AdmZip(filePath).extractAllTo(this.tmpFolder);
        const zipName = path.basename(filePath);
        // if the file is a not an incremental, reset the database schema
        if (zipName.charAt(4) !== "C") {
            await Promise.all(this.fileArray.map(file => this.setupSchema(file)));
            await this.createLastProcessedSchema();
        }
        if (this.files["CFA"] instanceof MultiRecordFile_1.MultiRecordFile) {
            await this.setLastScheduleId();
        }
        await Promise.all(fs.readdirSync(this.tmpFolder)
            .filter(filename => this.getFeedFile(filename))
            .map(filename => this.processFile(filename)));
        if (this.files["CFA"] instanceof MultiRecordFile_1.MultiRecordFile) {
            await this.removeOrphanStopTimes();
        }
        await this.updateLastFile(zipName);
    }
    /**
     * Drop and recreate the tables
     */
    async setupSchema(file) {
        await Promise.all(this.schemas(file).map(schema => schema.dropSchema()));
        await Promise.all(this.schemas(file).map(schema => schema.createSchema()));
    }
    /**
     * Create the last_file table (if it doesn't already exist)
     */
    async createLastProcessedSchema() {
        await this.db.query(`
      CREATE TABLE IF NOT EXISTS log ( 
        id INT(11) unsigned not null primary key auto_increment, 
        filename VARCHAR(255), 
        processed DATETIME 
      )
    `);
    }
    /**
     * Set the last schedule ID in the CFA record
     */
    async setLastScheduleId() {
        const [[lastSchedule]] = await this.db.query("SELECT id FROM schedule ORDER BY id desc LIMIT 1");
        const lastId = lastSchedule ? lastSchedule.id : 0;
        const cfaFile = this.files["CFA"];
        const bsRecord = cfaFile.records["BS"];
        bsRecord.lastId = lastId;
    }
    async removeOrphanStopTimes() {
        return Promise.all([
            this.db.query("DELETE FROM stop_time WHERE schedule NOT IN (SELECT id FROM schedule)"),
            this.db.query("DELETE FROM schedule_extra WHERE schedule NOT IN (SELECT id FROM schedule)")
        ]);
    }
    async updateLastFile(filename) {
        await this.db.query("INSERT INTO log VALUES (null, ?, NOW())", [filename]);
    }
    /**
     * Process the records inside the given file
     */
    async processFile(filename) {
        const file = this.getFeedFile(filename);
        const tables = await this.tables(file);
        const tableStream = new MySQLStream_1.MySQLStream(filename, file, tables);
        const stream = readFile(this.tmpFolder + filename).pipe(tableStream);
        try {
            await streamToPromise(stream);
            console.log(`Finished processing ${filename}`);
        }
        catch (err) {
            console.error(`Error processing ${filename}`);
            console.error(err);
        }
    }
    getFeedFile(filename) {
        return this.files[getExt(filename)];
    }
    schemas(file) {
        return file.recordTypes.map(record => new MySQLSchema_1.MySQLSchema(this.db, record));
    }
    async tables(file) {
        const index = {};
        for (const record of file.recordTypes) {
            if (!index[record.name]) {
                const db = record.orderedInserts ? await this.db.getConnection() : this.db;
                index[record.name] = new MySQLTable_1.MySQLTable(db, record.name);
            }
        }
        return index;
    }
    /**
     * Close the underling database connection
     */
    end() {
        return this.db.end();
    }
}
exports.ImportFeedCommand = ImportFeedCommand;
__decorate([
    memoize
], ImportFeedCommand.prototype, "getFeedFile", null);
__decorate([
    memoize
], ImportFeedCommand.prototype, "schemas", null);
__decorate([
    memoize
], ImportFeedCommand.prototype, "tables", null);
//# sourceMappingURL=ImportFeedCommand.js.map