"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutputGTFSCommand = void 0;
const agency_1 = require("../../config/gtfs/agency");
const ApplyOverlays_1 = require("../gtfs/command/ApplyOverlays");
const MergeSchedules_1 = require("../gtfs/command/MergeSchedules");
const ApplyAssociations_1 = require("../gtfs/command/ApplyAssociations");
const CreateCalendar_1 = require("../gtfs/command/CreateCalendar");
const fs = require("fs");
const AddLateNightServices_1 = require("../gtfs/command/AddLateNightServices");
const streamToPromise = require("stream-to-promise");
class OutputGTFSCommand {
    constructor(repository, output) {
        this.repository = repository;
        this.output = output;
    }
    /**
     * Turn the timetable feed into GTFS files
     */
    async run(argv) {
        this.baseDir = argv[3] || "./";
        if (!fs.existsSync(this.baseDir)) {
            throw new Error(`Output path ${this.baseDir} does not exist.`);
        }
        const associationsP = this.repository.getAssociations();
        const scheduleResultsP = this.repository.getSchedules();
        const transfersP = this.copy(this.repository.getTransfers(), "transfers.txt");
        const stopsP = this.copy(this.repository.getStops(), "stops.txt");
        const agencyP = this.copy(agency_1.agencies, "agency.txt");
        const fixedLinksP = this.copy(this.repository.getFixedLinks(), "links.txt");
        const schedules = this.getSchedules(await associationsP, await scheduleResultsP);
        const [calendars, calendarDates, serviceIds] = (0, CreateCalendar_1.createCalendar)(schedules);
        const calendarP = this.copy(calendars, "calendar.txt");
        const calendarDatesP = this.copy(calendarDates, "calendar_dates.txt");
        const tripsP = this.copyTrips(schedules, serviceIds);
        await Promise.all([
            agencyP,
            transfersP,
            stopsP,
            calendarP,
            calendarDatesP,
            tripsP,
            fixedLinksP,
            this.repository.end(),
            this.output.end()
        ]);
    }
    /**
     * Map SQL records to a file
     */
    async copy(results, filename) {
        const rows = await results;
        const output = this.output.open(this.baseDir + filename);
        console.log("Writing " + filename);
        rows.forEach(row => output.write(row));
        output.end();
        return streamToPromise(output);
    }
    /**
     * trips.txt, stop_times.txt and routes.txt have interdependencies so they are written together
     */
    copyTrips(schedules, serviceIds) {
        console.log("Writing trips.txt, stop_times.txt and routes.txt");
        const trips = this.output.open(this.baseDir + "trips.txt");
        const stopTimes = this.output.open(this.baseDir + "stop_times.txt");
        const routeFile = this.output.open(this.baseDir + "routes.txt");
        const routes = {};
        for (const schedule of schedules) {
            if (schedule.stopTimes.length <= 1) {
                continue;
            }
            const route = schedule.toRoute();
            routes[route.route_short_name] = routes[route.route_short_name] || route;
            const routeId = routes[route.route_short_name].route_id;
            const serviceId = serviceIds[schedule.calendar.id];
            trips.write(schedule.toTrip(serviceId, routeId));
            schedule.stopTimes.forEach(r => stopTimes.write(r));
        }
        for (const route of Object.values(routes)) {
            routeFile.write(route);
        }
        trips.end();
        stopTimes.end();
        routeFile.end();
        return Promise.all([
            streamToPromise(trips),
            streamToPromise(stopTimes),
            streamToPromise(routeFile),
        ]);
    }
    getSchedules(associations, scheduleResults) {
        const processedAssociations = (0, ApplyOverlays_1.applyOverlays)(associations);
        const processedSchedules = (0, ApplyOverlays_1.applyOverlays)(scheduleResults.schedules, scheduleResults.idGenerator);
        const associatedSchedules = (0, ApplyAssociations_1.applyAssociations)(processedSchedules, processedAssociations, scheduleResults.idGenerator);
        const mergedSchedules = (0, MergeSchedules_1.mergeSchedules)(associatedSchedules);
        const schedules = (0, AddLateNightServices_1.addLateNightServices)(mergedSchedules, scheduleResults.idGenerator);
        return schedules;
    }
}
exports.OutputGTFSCommand = OutputGTFSCommand;
//# sourceMappingURL=OutputGTFSCommand.js.map