"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MySQLSchema = void 0;
const TextField_1 = require("../feed/field/TextField");
const IntField_1 = require("../feed/field/IntField");
const BooleanField_1 = require("../feed/field/BooleanField");
const DateField_1 = require("../feed/field/DateField");
const TimeField_1 = require("../feed/field/TimeField");
const DoubleField_1 = require("../feed/field/DoubleField");
const ForeignKeyField_1 = require("../feed/field/ForeignKeyField");
class MySQLSchema {
    constructor(db, record) {
        this.db = db;
        this.record = record;
    }
    /**
     * Create the schema for the given record
     */
    createSchema() {
        return this.db.query(this.getSchema());
    }
    /**
     * Drop the table
     */
    dropSchema() {
        return this.db.query(`DROP TABLE IF EXISTS \`${this.record.name}\``);
    }
    getSchema() {
        const id = "id INT(11) unsigned auto_increment NOT NULL PRIMARY KEY";
        const fields = "," + Object.entries(this.record.fields).map(MySQLSchema.getField).join(',');
        const unique = this.record.key.length === 0 ? "" : `, UNIQUE ${this.record.name}_key (${this.record.key.join(',')})`;
        const indexes = this.record.indexes.map(index => `, KEY ${index} (${index})`);
        const table = [id, fields, unique, ...indexes].join('');
        return `CREATE TABLE IF NOT EXISTS \`${this.record.name}\` (${table}) Engine=InnoDB`;
    }
    static getField(entry) {
        const [name, field] = entry;
        const type = MySQLSchema.getFieldType(field);
        const nullable = MySQLSchema.getNullStatement(field);
        return `\`${name}\` ${type} ${nullable}`;
    }
    static getFieldType(field) {
        if (field instanceof TextField_1.VariableLengthText)
            return `VARCHAR(${field.length})`;
        if (field instanceof TextField_1.TextField)
            return `CHAR(${field.length})`;
        if (field instanceof BooleanField_1.BooleanField)
            return `TINYINT(1) unsigned`;
        if (field instanceof DateField_1.ShortDateField)
            return `DATE`;
        if (field instanceof DateField_1.DateField)
            return `DATE`;
        if (field instanceof DateField_1.NullDateField)
            return `DATE`;
        if (field instanceof TimeField_1.TimeField)
            return `TIME`;
        if (field instanceof DoubleField_1.DoubleField)
            return `DOUBLE(${field.length}, ${field.decimalDigits}) unsigned`;
        if (field instanceof IntField_1.ZeroFillIntField)
            return `CHAR(${field.length})`;
        if (field instanceof IntField_1.IntField)
            return `${MySQLSchema.getIntType(field.length)} unsigned`;
        if (field instanceof ForeignKeyField_1.ForeignKeyField)
            return `INT(11) unsigned`;
        throw new Error("Unknown field type");
    }
    static getIntType(length) {
        if (length > 9)
            return `BIGINT(${length})`;
        if (length > 7)
            return `INT(${length})`;
        if (length > 4)
            return `MEDIUMINT(${length})`;
        if (length > 2)
            return `SMALLINT(${length})`;
        return `TINYINT(${length})`;
    }
    static getNullStatement(field) {
        return field.nullable ? 'DEFAULT NULL' : 'NOT NULL';
    }
}
exports.MySQLSchema = MySQLSchema;
//# sourceMappingURL=MySQLSchema.js.map