"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MySQLTable = void 0;
const Record_1 = require("../feed/record/Record");
/**
 * Stateful class that provides access to a MySQL table and acts as buffer for inserts.
 */
class MySQLTable {
    constructor(db, tableName, flushLimit = 5000) {
        this.db = db;
        this.tableName = tableName;
        this.flushLimit = flushLimit;
        this.buffer = {
            [Record_1.RecordAction.Insert]: [],
            [Record_1.RecordAction.Update]: [],
            [Record_1.RecordAction.Delete]: [],
            [Record_1.RecordAction.DelayedInsert]: [],
        };
    }
    /**
     * Insert the given row to the table
     */
    async apply(row) {
        this.buffer[row.action].push(row);
        // if it's a delayed insert, also add a delete entry
        if (row.action === Record_1.RecordAction.DelayedInsert) {
            this.buffer[Record_1.RecordAction.Delete].push(Object.assign(Object.assign({}, row), { action: Record_1.RecordAction.Delete }));
        }
        // only flush the buffer it its not a delayed insert (as they are flushed at the end)
        else if (this.buffer[row.action].length >= this.flushLimit) {
            return this.flush(row.action);
        }
    }
    /**
     * Flush the table
     */
    async flush(type) {
        const rows = this.buffer[type];
        if (rows.length > 0) {
            this.buffer[type] = [];
            return this.queryWithRetry(type, rows);
        }
    }
    /**
     * Flush and return all promises
     */
    async close() {
        await Promise.all([
            this.flush(Record_1.RecordAction.Delete),
            this.flush(Record_1.RecordAction.Update),
            this.flush(Record_1.RecordAction.Insert)
        ]);
        await this.flush(Record_1.RecordAction.DelayedInsert);
        if (this.db.release) {
            await this.db.release();
        }
    }
    /**
     * Query with retry. Sometimes locking errors occur
     */
    async queryWithRetry(type, rows, numRetries = 3) {
        try {
            await this.query(type, rows);
        }
        catch (err) {
            if (err.errno === 1213 && numRetries > 0) {
                return this.queryWithRetry(type, rows, numRetries - 1);
            }
            else {
                throw err;
            }
        }
    }
    async query(type, rows) {
        const rowValues = rows.map(r => Object.values(r.values));
        switch (type) {
            case Record_1.RecordAction.Insert:
            case Record_1.RecordAction.DelayedInsert:
                await this.db.query(`INSERT IGNORE INTO \`${this.tableName}\` VALUES ?`, [rowValues]);
                return;
            case Record_1.RecordAction.Update:
                await this.db.query(`REPLACE INTO \`${this.tableName}\` VALUES ?`, [rowValues]);
                return;
            case Record_1.RecordAction.Delete:
                await this.db.query(`DELETE FROM \`${this.tableName}\` WHERE (${this.getDeleteSQL(rows)})`, rows.flatMap(row => Object.values(row.keysValues)));
                return;
            default:
                throw new Error("Unknown record action: " + type);
        }
    }
    getDeleteSQL(rows) {
        return rows.map(row => Object.keys(row.keysValues).map(k => `\`${k}\` = ?`).join(" AND ")).join(") OR (");
    }
}
exports.MySQLTable = MySQLTable;
//# sourceMappingURL=MySQLTable.js.map