"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyAssociations = void 0;
const Association_1 = require("../native/Association");
const ScheduleCalendar_1 = require("../native/ScheduleCalendar");
/**
 * Iterate through the associations matching association schedules records with base schedules and applying the
 * association as a join or split.
 *
 * Splits prepend the base schedule (0, point of split) to the association schedule
 * Joins append the base schedule (point of split, end) to the association schedule
 */
function applyAssociations(schedulesByTuid, associationsIndex, idGenerator) {
    for (const associations of Object.values(associationsIndex)) {
        // for each association
        for (const association of associations) {
            // get the date range for the associated schedules
            const assocCalendar = association.dateIndicator === Association_1.DateIndicator.Next
                ? association.calendar.shiftForward()
                : association.calendar;
            // get the associated schedules inside the date range of the association
            for (const assocSchedule of findSchedules(schedulesByTuid[association.assocTUID] || [], assocCalendar)) {
                // get the date range for the target base schedule (same or previous day of associated schedule NOT the association)
                const baseCalendar = association.dateIndicator === Association_1.DateIndicator.Next
                    ? assocSchedule.calendar.shiftBackward()
                    : assocSchedule.calendar;
                // find the matching base record
                const baseSchedules = findSchedules(schedulesByTuid[association.baseTUID] || [], baseCalendar);
                if (baseSchedules.length > 0) {
                    const [replacement, ...associatedSchedules] = association.apply(baseSchedules[0], assocSchedule, idGenerator);
                    // add the merged base and associated schedule to the TUID index
                    (schedulesByTuid[replacement.tuid] = schedulesByTuid[replacement.tuid] || []).push(replacement);
                    // remove the original associated schedule and replace with any substitute schedules created
                    schedulesByTuid[assocSchedule.tuid].splice(schedulesByTuid[assocSchedule.tuid].indexOf(assocSchedule), 1, ...associatedSchedules);
                }
            }
        }
    }
    return schedulesByTuid;
}
exports.applyAssociations = applyAssociations;
/**
 * Return schedules that overlap with the given calendar
 */
function findSchedules(schedules, calendar) {
    return schedules.filter(schedule => calendar.getOverlap(schedule.calendar) !== ScheduleCalendar_1.OverlapType.None);
}
//# sourceMappingURL=ApplyAssociations.js.map