"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssociationType = exports.DateIndicator = exports.Association = void 0;
const Schedule_1 = require("./Schedule");
const moment = require("moment");
const Duration_1 = require("./Duration");
class Association {
    constructor(id, baseTUID, assocTUID, assocLocation, dateIndicator, assocType, calendar, stp) {
        this.id = id;
        this.baseTUID = baseTUID;
        this.assocTUID = assocTUID;
        this.assocLocation = assocLocation;
        this.dateIndicator = dateIndicator;
        this.assocType = assocType;
        this.calendar = calendar;
        this.stp = stp;
    }
    get tuid() {
        return this.baseTUID + "_" + this.assocTUID + "_";
    }
    get hash() {
        return this.tuid + this.assocLocation + this.calendar.binaryDays;
    }
    /**
     * Clone the association with a different calendar
     */
    clone(calendar, id) {
        return new Association(id, this.baseTUID, this.assocTUID, this.assocLocation, this.dateIndicator, this.assocType, calendar, this.stp);
    }
    /**
     * Apply the join or split to the associated schedule. Check for any days that the associated service runs but the
     * association does not and create additional schedules to cover those periods.
     */
    apply(base, assoc, idGenerator) {
        const assocCalendar = this.dateIndicator === DateIndicator.Next ? this.calendar.shiftForward() : this.calendar;
        const schedules = [this.mergeSchedules(base, assoc)];
        // if the associated train starts running before the association, clone the associated schedule for those dates
        if (assoc.calendar.runsFrom.isBefore(assocCalendar.runsFrom)) {
            const before = assoc.calendar.clone(assoc.calendar.runsFrom, assocCalendar.runsFrom.clone().subtract(1, "days"));
            schedules.push(assoc.clone(before, idGenerator.next().value));
        }
        // if the associated train runs after the association has finished, clone the associated schedule for those dates
        if (assoc.calendar.runsTo.isAfter(assocCalendar.runsTo)) {
            const after = assoc.calendar.clone(assocCalendar.runsTo.clone().add(1, "days"), assoc.calendar.runsTo);
            schedules.push(assoc.clone(after, idGenerator.next().value));
        }
        // for each exclude day of the association
        for (const excludeDay of Object.values(assocCalendar.excludeDays)) {
            schedules.push(assoc.clone(assoc.calendar.clone(excludeDay, excludeDay), idGenerator.next().value));
        }
        return schedules;
    }
    /**
     * Apply the split or join to the given schedules
     */
    mergeSchedules(base, assoc) {
        let tuid;
        let start;
        let assocStop;
        let end;
        const baseStopTime = base.stopAt(this.assocLocation);
        const assocStopTime = assoc.stopAt(this.assocLocation);
        // this should never happen, unless data feed is corrupted. It will prevent us from update failure
        if (baseStopTime === undefined || assocStopTime === undefined) {
            return assoc;
        }
        if (this.assocType === AssociationType.Split) {
            tuid = base.tuid + "_" + assoc.tuid;
            start = base.before(this.assocLocation);
            assocStop = this.mergeAssociationStop(baseStopTime, assocStopTime);
            end = assoc.after(this.assocLocation);
        }
        else {
            tuid = assoc.tuid + "_" + base.tuid;
            start = assoc.before(this.assocLocation);
            assocStop = this.mergeAssociationStop(assocStopTime, baseStopTime);
            end = base.after(this.assocLocation);
        }
        let stopSequence = 1;
        const stops = [
            ...start.map(s => cloneStop(s, stopSequence++, assoc.id)),
            cloneStop(assocStop, stopSequence++, assoc.id),
            ...end.map(s => cloneStop(s, stopSequence++, assoc.id, assocStop))
        ];
        const calendar = this.dateIndicator === DateIndicator.Next ? assoc.calendar.shiftBackward() : assoc.calendar;
        return new Schedule_1.Schedule(assoc.id, stops, tuid, assoc.rsid, 
        // only take the part of the schedule that the association applies to
        calendar.clone(moment.max(this.calendar.runsFrom, calendar.runsFrom), moment.min(this.calendar.runsTo, calendar.runsTo)), assoc.mode, assoc.operator, assoc.stp, assoc.firstClassAvailable, assoc.reservationPossible);
    }
    /**
     * Take the arrival time of the first stop and the departure time of the second stop and put them into a new stop
     */
    mergeAssociationStop(arrivalStop, departureStop) {
        let arrivalTime = moment.duration(arrivalStop.arrival_time);
        let departureTime = moment.duration(departureStop.departure_time);
        if (arrivalTime.asSeconds() > departureTime.asSeconds()) {
            if (this.dateIndicator === DateIndicator.Next) {
                departureTime.add(1, "days");
            }
            else {
                arrivalTime = moment.duration(departureStop.arrival_time);
            }
        }
        return Object.assign({}, arrivalStop, {
            arrival_time: (0, Duration_1.formatDuration)(arrivalTime.asSeconds()),
            departure_time: (0, Duration_1.formatDuration)(departureTime.asSeconds()),
            pickup_type: departureStop.pickup_type,
            drop_off_type: arrivalStop.drop_off_type
        });
    }
}
exports.Association = Association;
/**
 * Clone the given stop overriding the sequence number and modifying the arrival/departure times if necessary
 */
function cloneStop(stop, stopSequence, tripId, assocStop = null) {
    const assocTime = moment.duration(assocStop && assocStop.arrival_time ? assocStop.arrival_time : "00:00");
    const departureTime = stop.departure_time ? moment.duration(stop.departure_time) : undefined;
    if (departureTime && departureTime.asSeconds() < assocTime.asSeconds()) {
        departureTime.add(1, "day");
    }
    const arrivalTime = stop.arrival_time ? moment.duration(stop.arrival_time) : undefined;
    if (arrivalTime && arrivalTime.asSeconds() < assocTime.asSeconds()) {
        arrivalTime.add(1, "day");
    }
    return Object.assign({}, stop, {
        arrival_time: arrivalTime ? (0, Duration_1.formatDuration)(arrivalTime.asSeconds()) : undefined,
        departure_time: departureTime ? (0, Duration_1.formatDuration)(departureTime.asSeconds()) : undefined,
        stop_sequence: stopSequence,
        trip_id: tripId
    });
}
var DateIndicator;
(function (DateIndicator) {
    DateIndicator["Same"] = "S";
    DateIndicator["Next"] = "N";
    DateIndicator["Previous"] = "P";
})(DateIndicator || (exports.DateIndicator = DateIndicator = {}));
var AssociationType;
(function (AssociationType) {
    AssociationType["Split"] = "VV";
    AssociationType["Join"] = "JJ";
    AssociationType["NA"] = "";
})(AssociationType || (exports.AssociationType = AssociationType = {}));
//# sourceMappingURL=Association.js.map