"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Schedule = void 0;
const Route_1 = require("../file/Route");
/**
 * A CIF schedule (BS record)
 */
class Schedule {
    constructor(id, stopTimes, tuid, rsid, calendar, mode, operator, stp, firstClassAvailable, reservationPossible) {
        this.id = id;
        this.stopTimes = stopTimes;
        this.tuid = tuid;
        this.rsid = rsid;
        this.calendar = calendar;
        this.mode = mode;
        this.operator = operator;
        this.stp = stp;
        this.firstClassAvailable = firstClassAvailable;
        this.reservationPossible = reservationPossible;
    }
    get origin() {
        return this.stopTimes[0].stop_id;
    }
    get destination() {
        return this.stopTimes[this.stopTimes.length - 1].stop_id;
    }
    get hash() {
        return this.tuid + this.stopTimes.map(s => s.stop_id + s.departure_time + s.arrival_time).join("") + this.calendar.binaryDays;
    }
    /**
     * Clone the current record with the new calendar and id
     */
    clone(calendar, scheduleId) {
        return new Schedule(scheduleId, this.stopTimes.map(st => Object.assign({}, st, { trip_id: scheduleId })), this.tuid, this.rsid, calendar, this.mode, this.operator, this.stp, this.firstClassAvailable, this.reservationPossible);
    }
    /**
     * Convert to a GTFS Trip
     */
    toTrip(serviceId, routeId) {
        return {
            route_id: routeId,
            service_id: serviceId,
            trip_id: this.id,
            trip_headsign: this.tuid,
            trip_short_name: this.rsid,
            direction_id: 0,
            wheelchair_accessible: 1,
            bikes_allowed: 0
        };
    }
    /**
     * Convert to GTFS Route
     */
    toRoute() {
        return {
            route_id: this.id,
            agency_id: this.operator || "ZZ",
            route_short_name: `${this.operator || "Z"}:${this.origin}->${this.destination}:${this.mode}`,
            route_long_name: `${this.operator || "Z"} ${this.modeDescription.toLowerCase()} service from ${this.origin} to ${this.destination}`,
            route_type: this.mode,
            route_text_color: null,
            route_color: null,
            route_url: null,
            route_desc: [this.modeDescription, this.classDescription, this.reservationDescription].join(". ")
        };
    }
    get modeDescription() {
        switch (this.mode) {
            case Route_1.RouteType.Rail: return "Train";
            case Route_1.RouteType.Subway: return "Underground";
            case Route_1.RouteType.Tram: return "Tram";
            case Route_1.RouteType.Bus: return "Bus";
            case Route_1.RouteType.ReplacementBus: return "Replacement bus";
            case Route_1.RouteType.Ferry: return "Boat";
            default: return "Train";
        }
    }
    get classDescription() {
        return this.firstClassAvailable ? "First class available" : "Standard class only";
    }
    get reservationDescription() {
        return this.reservationPossible ? "Reservation possible" : "Reservation not possible";
    }
    before(location) {
        return this.stopTimes.slice(0, this.stopTimes.findIndex(s => s.stop_id === location));
    }
    after(location) {
        return this.stopTimes.slice(this.stopTimes.findIndex(s => s.stop_id === location) + 1);
    }
    stopAt(location) {
        return this.stopTimes.find(s => s.stop_id === location);
    }
}
exports.Schedule = Schedule;
//# sourceMappingURL=Schedule.js.map