"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NO_DAYS = exports.OverlapType = exports.ScheduleCalendar = void 0;
const memoize = require("memoized-class-decorator");
const moment = require("moment");
class ScheduleCalendar {
    constructor(runsFrom, runsTo, days, excludeDays = {}) {
        this.runsFrom = runsFrom;
        this.runsTo = runsTo;
        this.days = days;
        this.excludeDays = excludeDays;
    }
    get id() {
        return this.runsFrom.format("YYYYMMDD") + this.runsTo.format("YYYYMMDD") + this.binaryDays + Object.keys(this.excludeDays).join("");
    }
    get binaryDays() {
        return parseInt(Object.values(this.days).join(""), 2);
    }
    /**
     * Count the number of days that the overlay shares with this schedule and return true if the max has been exceeded
     */
    getOverlap(overlay) {
        // if there are no overlapping days
        if ((this.binaryDays & overlay.binaryDays) === 0) {
            return OverlapType.None;
        }
        let numDays = 0;
        for (const sharedDay of this.sharedDays(overlay)) {
            const key = sharedDay.format("YYYYMMDD");
            const isShared = !this.excludeDays[key] && !overlay.excludeDays[key];
            if (isShared && ++numDays > ScheduleCalendar.SHORT_OVERLAY_LENGTH) {
                return OverlapType.Long;
            }
        }
        return (numDays > 0) ? OverlapType.Short : OverlapType.None;
    }
    /**
     * Add each date in the range as an exclude day
     */
    addExcludeDays(overlay) {
        const excludeDays = Object.assign({}, this.excludeDays); // clone
        for (const sharedDay of this.sharedDays(overlay)) {
            excludeDays[sharedDay.format("YYYYMMDD")] = sharedDay;
        }
        const calendar = this.clone(this.runsFrom, this.runsTo, exports.NO_DAYS, excludeDays);
        return calendar.runsFrom.isSameOrBefore(calendar.runsTo) ? [calendar] : [];
    }
    /**
     * Returns the overlapping days between schedules (does not account for exclude days)
     */
    *sharedDays(overlay) {
        const startDate = moment.max(this.runsFrom, overlay.runsFrom).clone();
        const endDate = moment.min(this.runsTo, overlay.runsTo);
        while (startDate.isSameOrBefore(endDate)) {
            if (this.days[startDate.day()] && overlay.days[startDate.day()]) {
                yield startDate.clone();
            }
            startDate.add(1, "days");
        }
    }
    /**
     * Remove the given date range from this schedule and return one or two calendars
     */
    divideAround(calendar) {
        const calendars = [
            this.clone(this.runsFrom.clone(), calendar.runsFrom.clone().subtract(1, "days")),
            this.clone(calendar.runsTo.clone().add(1, "days"), this.runsTo.clone())
        ];
        // if there are any days left after applying the overlay
        if (this.binaryDays - (this.binaryDays & calendar.binaryDays) > 0) {
            calendars.push(this.clone(calendar.runsFrom.clone(), calendar.runsTo.clone(), calendar.days, this.excludeDays));
        }
        return calendars.filter(c => c.runsFrom.isSameOrBefore(c.runsTo));
    }
    /**
     * Remove the given days from the calendar then tighten the dates
     */
    clone(start, end, removeDays = exports.NO_DAYS, excludeDays = this.excludeDays) {
        const days = this.removeDays(removeDays);
        // skip forward to the first day the schedule is operating
        while (days[start.day()] === 0 || excludeDays[start.format("YYYYMMDD")] && start.isSameOrBefore(end)) {
            start.add(1, "days");
        }
        // skip backward to the first day the schedule is operating
        while (days[end.day()] === 0 || excludeDays[end.format("YYYYMMDD")] && end.isSameOrAfter(start)) {
            end.subtract(1, "days");
        }
        const newExcludes = Object
            .values(excludeDays)
            .filter(d => d.isBetween(start, end, "days", "[]"))
            .reduce((days, day) => { days[day.format("YYYYMMDD")] = day; return days; }, {});
        return new ScheduleCalendar(start, end, days, newExcludes);
    }
    removeDays(days) {
        return {
            0: this.days[0] && !days[0] ? 1 : 0,
            1: this.days[1] && !days[1] ? 1 : 0,
            2: this.days[2] && !days[2] ? 1 : 0,
            3: this.days[3] && !days[3] ? 1 : 0,
            4: this.days[4] && !days[4] ? 1 : 0,
            5: this.days[5] && !days[5] ? 1 : 0,
            6: this.days[6] && !days[6] ? 1 : 0
        };
    }
    /**
     * Convert to a GTFS Calendar object
     */
    toCalendar(serviceId) {
        return {
            service_id: serviceId,
            monday: this.days[1],
            tuesday: this.days[2],
            wednesday: this.days[3],
            thursday: this.days[4],
            friday: this.days[5],
            saturday: this.days[6],
            sunday: this.days[0],
            start_date: this.runsFrom.format("YYYYMMDD"),
            end_date: this.runsTo.format("YYYYMMDD"),
        };
    }
    /**
     * Convert exclude days to GTFS Calendar Dates
     */
    toCalendarDates(serviceId) {
        return Object.values(this.excludeDays).map(d => {
            return {
                service_id: serviceId,
                date: d.format("YYYYMMDD"),
                exception_type: 2
            };
        });
    }
    /**
     * Returns true if this calendar would not be valid on any days before the given calendar starts
     */
    canMerge(calendar) {
        const startDate = this.runsTo.clone();
        let numAdditionalExcludeDays = 0;
        while (startDate.add(1, "days").isBefore(calendar.runsFrom)) {
            if (this.days[startDate.day()] && ++numAdditionalExcludeDays > ScheduleCalendar.SHORT_OVERLAY_LENGTH) {
                return false;
            }
        }
        return true;
    }
    /**
     * Return a new calendar starting from the runsFrom of this calendar and running to the runsTo of the given calendar.
     *
     * Exclude days are merged together.
     */
    merge(calendar) {
        const excludeDays = Object.assign({}, calendar.excludeDays, this.excludeDays);
        const startDate = this.runsTo.clone();
        while (startDate.add(1, "days").isBefore(calendar.runsFrom)) {
            if (this.days[startDate.day()]) {
                excludeDays[startDate.format("YYYYMMDD")] = startDate.clone();
            }
        }
        // for any shared
        for (const sharedDay of this.sharedDays(calendar)) {
            const key = sharedDay.format("YYYYMMDD");
            // if the shared day is only excluded in one overlay, remove it
            if (!(this.excludeDays[key] && calendar.excludeDays[key])) {
                delete excludeDays[key];
            }
        }
        return new ScheduleCalendar(this.runsFrom, moment.max(this.runsTo, calendar.runsTo), this.days, excludeDays);
    }
    /**
     * Shift the calendar forward a day
     */
    shiftForward() {
        const excludeDays = {};
        for (const day of Object.values(this.excludeDays)) {
            const shiftedDay = day.clone().add(1, "days");
            excludeDays[shiftedDay.format("YYYYMMDD")] = shiftedDay;
        }
        return new ScheduleCalendar(this.runsFrom.clone().add(1, "days"), this.runsTo.clone().add(1, "days"), {
            0: this.days[6],
            1: this.days[0],
            2: this.days[1],
            3: this.days[2],
            4: this.days[3],
            5: this.days[4],
            6: this.days[5],
        }, excludeDays);
    }
    /**
     * Shift the calendar back a day
     */
    shiftBackward() {
        const excludeDays = {};
        for (const day of Object.values(this.excludeDays)) {
            const shiftedDay = day.clone().subtract(1, "days");
            excludeDays[shiftedDay.format("YYYYMMDD")] = shiftedDay;
        }
        return new ScheduleCalendar(this.runsFrom.clone().subtract(1, "days"), this.runsTo.clone().subtract(1, "days"), {
            0: this.days[1],
            1: this.days[2],
            2: this.days[3],
            3: this.days[4],
            4: this.days[5],
            5: this.days[6],
            6: this.days[0],
        }, excludeDays);
    }
}
exports.ScheduleCalendar = ScheduleCalendar;
ScheduleCalendar.SHORT_OVERLAY_LENGTH = 7;
__decorate([
    memoize
], ScheduleCalendar.prototype, "id", null);
__decorate([
    memoize
], ScheduleCalendar.prototype, "binaryDays", null);
__decorate([
    memoize
], ScheduleCalendar.prototype, "shiftForward", null);
__decorate([
    memoize
], ScheduleCalendar.prototype, "shiftBackward", null);
var OverlapType;
(function (OverlapType) {
    OverlapType[OverlapType["None"] = 0] = "None";
    OverlapType[OverlapType["Short"] = 1] = "Short";
    OverlapType[OverlapType["Long"] = 2] = "Long";
})(OverlapType || (exports.OverlapType = OverlapType = {}));
exports.NO_DAYS = { 0: 0, 1: 0, 2: 0, 3: 0, 4: 0, 5: 0, 6: 0 };
//# sourceMappingURL=ScheduleCalendar.js.map