"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CIFRepository = void 0;
const proj4 = require("proj4");
const moment = require("moment");
const ScheduleCalendar_1 = require("../native/ScheduleCalendar");
const Association_1 = require("../native/Association");
const ScheduleBuilder_1 = require("./ScheduleBuilder");
/**
 * Provide access to the CIF/TTIS data in a vaguely GTFS-ish shape.
 */
class CIFRepository {
    constructor(db, stream, stationCoordinates) {
        this.db = db;
        this.stream = stream;
        this.stationCoordinates = stationCoordinates;
        proj4.defs('EPSG:27700', '+proj=tmerc +lat_0=49 +lon_0=-2 +k=0.9996012717 +x_0=400000 +y_0=-100000 +ellps=airy +datum=OSGB36 +units=m +no_defs');
    }
    /**
     * Return the interchange time between each station
     */
    async getTransfers() {
        const [results] = await this.db.query(`
      SELECT 
        crs_code AS from_stop_id, 
        crs_code AS to_stop_id, 
        2 AS transfer_type, 
        minimum_change_time * 60 AS min_transfer_time 
      FROM physical_station WHERE cate_interchange_status IS NOT NULL
      GROUP BY crs_code
    `);
        return results;
    }
    /**
     * Return all the stops with some configurable long/lat applied
     */
    async getStops() {
        const [results] = await this.db.query(`
      SELECT
        crs_code AS stop_id, 
        tiploc_code AS stop_code,
        station_name AS stop_name,
        cate_interchange_status AS stop_desc,
        NULL AS zone_id,
        NULL AS stop_url,
        NULL AS location_type,
        NULL AS parent_station,
        IF(POSITION("(CIE" IN station_name), "Europe/Dublin", "Europe/London") AS stop_timezone,
        0 AS wheelchair_boarding,
        easting,
        northing
      FROM physical_station WHERE crs_code IS NOT NULL
      GROUP BY crs_code
    `);
        // overlay the long and latitude values from configuration
        return results.map(row => {
            const [stop_lon, stop_lat] = proj4('EPSG:27700', 'EPSG:4326', [(row.easting - 10000) * 100, (row.northing - 60000) * 100]);
            const _a = Object.assign(Object.assign({}, row), { stop_lon, stop_lat }), { easting, northing } = _a, stop = __rest(_a, ["easting", "northing"]);
            return Object.assign(stop, this.stationCoordinates[stop.stop_id]);
        });
    }
    /**
     * Return the schedules and z trains. These queries probably require some explanation:
     *
     * The first query selects the stop times for all passenger services between now and + 3 months. It's important that
     * the stop time location is mapped to physical stations to avoid getting fake CRS codes from the tiploc data.
     *
     * The second query selects all the z-trains (usually replacement buses) within three months. They already use CRS
     * codes as the location so avoid the disaster above.
     */
    async getSchedules() {
        const scheduleBuilder = new ScheduleBuilder_1.ScheduleBuilder();
        const [[lastSchedule]] = await this.db.query("SELECT id FROM schedule ORDER BY id desc LIMIT 1");
        await Promise.all([
            scheduleBuilder.loadSchedules(this.stream.query(`
        SELECT
          schedule.id AS id, train_uid, retail_train_id, runs_from, runs_to,
          monday, tuesday, wednesday, thursday, friday, saturday, sunday,
          crs_code, stp_indicator, public_arrival_time, public_departure_time,
          IF(train_status="S", "SS", train_category) AS train_category,
          scheduled_arrival_time AS scheduled_arrival_time,
          scheduled_departure_time AS scheduled_departure_time,
          platform, atoc_code, stop_time.id AS stop_id, activity, reservations, train_class
        FROM schedule
        LEFT JOIN schedule_extra ON schedule.id = schedule_extra.schedule
        LEFT JOIN stop_time ON schedule.id = stop_time.schedule
        LEFT JOIN physical_station ps ON location = ps.tiploc_code
        WHERE
        (
          stop_time.id IS NULL OR crs_code IS NOT NULL
        )
        AND runs_from < CURDATE() + INTERVAL 3 MONTH
        AND runs_to >= CURDATE()
        AND scheduled_pass_time is null
        ORDER BY stp_indicator DESC, id, stop_id
      `)),
            scheduleBuilder.loadSchedules(this.stream.query(`
        SELECT
          ${lastSchedule.id} + z_schedule.id AS id, train_uid, null, runs_from, runs_to,
          monday, tuesday, wednesday, thursday, friday, saturday, sunday,
          stp_indicator, location AS crs_code, train_category,
          public_arrival_time, public_departure_time, scheduled_arrival_time, scheduled_departure_time,
          platform, atoc_code, z_stop_time.id AS stop_id, activity, NULL AS reservations, "S" AS train_class 
        FROM z_schedule
        LEFT JOIN z_schedule_extra ON z_schedule.id = z_schedule_extra.schedule
        JOIN z_stop_time ON z_schedule.id = z_stop_time.z_schedule
        WHERE runs_from < CURDATE() + INTERVAL 3 MONTH
        AND runs_to >= CURDATE()
        ORDER BY stop_id
      `))
        ]);
        return scheduleBuilder.results;
    }
    /**
     * Get associations
     */
    async getAssociations() {
        const [results] = await this.db.query(`
      SELECT 
        a.id AS id, base_uid, assoc_uid, crs_code, assoc_date_ind, assoc_cat,
        monday, tuesday, wednesday, thursday, friday, saturday, sunday,
        start_date, end_date, stp_indicator
      FROM association a
      JOIN tiploc ON assoc_location = tiploc_code
      WHERE start_date < CURDATE() + INTERVAL 3 MONTH
      AND end_date >= CURDATE()
      ORDER BY stp_indicator DESC, id
    `);
        return results.map(row => new Association_1.Association(row.id, row.base_uid, row.assoc_uid, row.crs_code, row.assoc_date_ind, row.assoc_cat, new ScheduleCalendar_1.ScheduleCalendar(moment(row.start_date), moment(row.end_date), {
            0: row.sunday,
            1: row.monday,
            2: row.tuesday,
            3: row.wednesday,
            4: row.thursday,
            5: row.friday,
            6: row.saturday
        }), row.stp_indicator));
    }
    /**
     * Return the ALF information
     */
    async getFixedLinks() {
        // use the additional fixed links if possible and fill the missing data with fixed_links
        const [rows] = await this.db.query(`
      SELECT
        mode, duration * 60 as duration, origin, destination,
        start_time, end_time, start_date, end_date,
        monday, tuesday, wednesday, thursday, friday, saturday, sunday
      FROM additional_fixed_link
      WHERE origin IN (SELECT crs_code FROM physical_station)
      AND destination IN (SELECT crs_code FROM physical_station)
      UNION
      SELECT
        mode, duration * 60 as duration, origin, destination,
        "00:00:00", "23:59:59", "2017-01-01", "2038-01-19",
        1,1,1,1,1,1,1
      FROM fixed_link
      WHERE CONCAT(origin, destination) NOT IN (
        SELECT CONCAT(origin, destination) FROM additional_fixed_link
      )
    `);
        const results = [];
        for (const row of rows) {
            results.push(this.getFixedLinkRow(row.origin, row.destination, row));
            results.push(this.getFixedLinkRow(row.destination, row.origin, row));
        }
        return results;
    }
    getFixedLinkRow(origin, destination, row) {
        return {
            from_stop_id: origin,
            to_stop_id: destination,
            mode: row.mode,
            duration: row.duration,
            start_time: row.start_time,
            end_time: row.end_time,
            start_date: (row.start_date || "2017-01-01"),
            end_date: (row.end_date || "2038-01-19"),
            monday: row.monday,
            tuesday: row.tuesday,
            wednesday: row.wednesday,
            thursday: row.thursday,
            friday: row.friday,
            saturday: row.saturday,
            sunday: row.sunday
        };
    }
    /**
     * Close the underlying database
     */
    end() {
        return Promise.all([this.db.end(), this.stream.end()]);
    }
}
exports.CIFRepository = CIFRepository;
var FixedLinkMode;
(function (FixedLinkMode) {
    FixedLinkMode["Walk"] = "WALK";
    FixedLinkMode["Metro"] = "METRO";
    FixedLinkMode["Transfer"] = "TRANSFER";
    FixedLinkMode["Tube"] = "TUBE";
    FixedLinkMode["Bus"] = "BUS";
})(FixedLinkMode || (FixedLinkMode = {}));
//# sourceMappingURL=CIFRepository.js.map