"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScheduleBuilder = void 0;
const OverlayRecord_1 = require("../native/OverlayRecord");
const Schedule_1 = require("../native/Schedule");
const Route_1 = require("../file/Route");
const moment = require("moment");
const ScheduleCalendar_1 = require("../native/ScheduleCalendar");
const pickupActivities = ["T ", "TB", "U "];
const dropOffActivities = ["T ", "TF", "D "];
const coordinatedActivity = ["R "];
const notAdvertised = "N ";
/**
 * This class takes a stream of results and builds a list of Schedules
 */
class ScheduleBuilder {
    constructor() {
        this.schedules = [];
        this.maxId = 0;
    }
    /**
     * Take a stream of ScheduleStopTimeRow, turn them into Schedule objects and add the result to the schedules
     */
    loadSchedules(results) {
        return new Promise((resolve, reject) => {
            let stops = [];
            let prevRow;
            let departureHour = 4;
            results.on("result", (row) => {
                if (prevRow && prevRow.id !== row.id) {
                    this.schedules.push(this.createSchedule(prevRow, stops));
                    stops = [];
                    departureHour = row.public_arrival_time
                        ? parseInt(row.public_arrival_time.substr(0, 2), 10)
                        : row.public_departure_time ? parseInt(row.public_departure_time.substr(0, 2), 10) : 4;
                }
                if (row.stp_indicator !== OverlayRecord_1.STP.Cancellation) {
                    const stop = this.createStop(row, stops.length + 1, departureHour);
                    if (prevRow && prevRow.id === row.id && row.crs_code === prevRow.crs_code) {
                        if (stop.pickup_type === 0 || stop.drop_off_type === 0) {
                            stops[stops.length - 1] = Object.assign(stop, { stop_sequence: stops.length });
                        }
                    }
                    else {
                        stops.push(stop);
                    }
                }
                prevRow = row;
            });
            results.on("end", () => {
                if (prevRow) {
                    this.schedules.push(this.createSchedule(prevRow, stops));
                }
                resolve();
            });
            results.on("error", reject);
        });
    }
    createSchedule(row, stops) {
        this.maxId = Math.max(this.maxId, row.id);
        const mode = routeTypeIndex.hasOwnProperty(row.train_category) ? routeTypeIndex[row.train_category] : Route_1.RouteType.Rail;
        return new Schedule_1.Schedule(row.id, stops, row.train_uid, row.retail_train_id, new ScheduleCalendar_1.ScheduleCalendar(moment(row.runs_from), moment(row.runs_to), {
            0: row.sunday,
            1: row.monday,
            2: row.tuesday,
            3: row.wednesday,
            4: row.thursday,
            5: row.friday,
            6: row.saturday
        }), mode, row.atoc_code, row.stp_indicator, mode === Route_1.RouteType.Rail && row.train_class !== "S", row.reservations !== null);
    }
    createStop(row, stopId, departHour) {
        let arrivalTime, departureTime;
        // if either public time is set, use those
        if (row.public_arrival_time || row.public_departure_time) {
            arrivalTime = this.formatTime(row.public_arrival_time, departHour);
            departureTime = this.formatTime(row.public_departure_time, departHour);
        }
        // if no public time at all (no set down or pick) use the scheduled time
        else {
            arrivalTime = this.formatTime(row.scheduled_arrival_time, departHour);
            departureTime = this.formatTime(row.scheduled_departure_time, departHour);
        }
        const activities = row.activity.match(/.{1,2}/g) || [];
        const pickup = pickupActivities.find(a => activities.includes(a)) && !activities.includes(notAdvertised) ? 0 : 1;
        const coordinatedDropOff = coordinatedActivity.find(a => activities.includes(a)) ? 3 : 0;
        const dropOff = dropOffActivities.find(a => activities.includes(a)) ? 0 : 1;
        return {
            trip_id: row.id,
            arrival_time: (arrivalTime || departureTime),
            departure_time: (departureTime || arrivalTime),
            stop_id: row.crs_code,
            stop_sequence: stopId,
            stop_headsign: row.platform,
            pickup_type: coordinatedDropOff || pickup,
            drop_off_type: coordinatedDropOff || dropOff,
            shape_dist_traveled: null,
            timepoint: 1
        };
    }
    formatTime(time, originDepartureHour) {
        if (time === null)
            return null;
        const departureHour = parseInt(time.substr(0, 2), 10);
        // if the service started after 4am and after the current stops departure hour we've probably rolled over midnight
        if (originDepartureHour >= 4 && originDepartureHour > departureHour) {
            return (departureHour + 24) + time.substr(2);
        }
        return time;
    }
    get results() {
        return {
            schedules: this.schedules,
            idGenerator: this.getIdGenerator(this.maxId)
        };
    }
    *getIdGenerator(startId) {
        let id = startId + 1;
        while (true) {
            yield id++;
        }
    }
}
exports.ScheduleBuilder = ScheduleBuilder;
const routeTypeIndex = {
    "OO": Route_1.RouteType.Rail,
    "XX": Route_1.RouteType.Rail,
    "XZ": Route_1.RouteType.Rail,
    "BR": Route_1.RouteType.ReplacementBus,
    "BS": Route_1.RouteType.Bus,
    "OL": Route_1.RouteType.Subway,
    "XC": Route_1.RouteType.Rail,
    "SS": Route_1.RouteType.Ferry
};
//# sourceMappingURL=ScheduleBuilder.js.map